import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class EnterpriseControlPanel extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = -280782189989505766L;
	private Enterprise enterprise;
	private JToggleButton btnShield;
	private JButton btnAtackNearestKlingon;
	private JButton btnAtackAllKlingon;
	private JButton btnStopAttack; 
	private JList<SpaceShip> list;
	private final ButtonGroup buttonGroupMove = new ButtonGroup();
	private final ButtonGroup buttonGroupEngine = new ButtonGroup();
	private JToggleButton btnStandBy;
	private JToggleButton btnRescue;
	private JButton tglbtnLeft;
	private JButton tglbtnRight;
	private JButton tglbtnForward;
	private JToggleButton tglbtnStop;
	private JPanel panelManeuvers;
	private JPanel panelManualControl;
	private JPanel panelMove;
	private JPanel panelAttack;

	/**
	 * Create the panel.
	 */
	public EnterpriseControlPanel() {
		getContentPane().setBackground(Color.BLACK);
		try{ 
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsClassicLookAndFeel");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		initComponents();
		recreateManeuversButtons();
		pack();
	}
	
	protected void initComponents(){
		GridBagLayout gridBagLayout = new GridBagLayout();
		getContentPane().setLayout(gridBagLayout);

		btnShield = new JToggleButton("Shield [SPACE]");
		btnShield.setForeground(new Color(255, 204, 0));
		btnShield.setBackground(new Color(51, 102, 255));
		btnShield.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (enterprise != null) {
					enterprise.setShieldOn(getBtnShield().getModel()
							.isSelected());
				}
			}
		});
		GridBagConstraints gbc_btnShield = new GridBagConstraints();
		gbc_btnShield.gridx = GridBagConstraints.RELATIVE;
		gbc_btnShield.gridy = GridBagConstraints.RELATIVE;
		gbc_btnShield.fill = GridBagConstraints.BOTH;
		gbc_btnShield.anchor = GridBagConstraints.CENTER;
		gbc_btnShield.weighty = 1;
		getContentPane().add(btnShield, gbc_btnShield);

		GridBagConstraints gbc_panelAttack = new GridBagConstraints();
		gbc_panelAttack.anchor = GridBagConstraints.CENTER;
		gbc_panelAttack.insets = new Insets(0, 0, 0, 5);
		gbc_panelAttack.gridx = GridBagConstraints.RELATIVE;
		gbc_panelAttack.gridy = GridBagConstraints.RELATIVE;
		gbc_panelAttack.gridwidth = GridBagConstraints.REMAINDER;
		gbc_panelAttack.gridheight = 2;
		gbc_panelAttack.weightx = 1;
		gbc_panelAttack.weighty = 1;
		gbc_panelAttack.fill = GridBagConstraints.BOTH;
		gbc_panelAttack.anchor = GridBagConstraints.CENTER;
		getContentPane().add(getPanelAttack(), gbc_panelAttack);

		GridBagConstraints gbc_panelMove = new GridBagConstraints();
		gbc_panelMove.insets = new Insets(0, 0, 0, 5);
		gbc_panelMove.gridx = 0;
		gbc_panelMove.gridy = 1;
		gbc_panelMove.weighty = 3;
		gbc_panelMove.fill = GridBagConstraints.BOTH;
		getContentPane().add(getPanelMove(), gbc_panelMove);


	}

	public Enterprise getEnterprise() {
		return enterprise;
	}

	public void setEnterprise(Enterprise enterprise) {
		this.enterprise = enterprise;
		this.setTitle(enterprise.getName());
		recreateManeuversButtons();
		updateState();

	}

	class ManeuverAction extends AbstractAction {
		private static final long serialVersionUID = 253554674398217621L;
		int index;

		public ManeuverAction(int index) {
			this.index = index;
			this.putValue(Action.NAME, enterprise.getManeuverName(index));
			String number = Integer.toString(index + 1);
			if (number.length() == 1) {
				this.putValue(Action.NAME, enterprise.getManeuverName(index) + "[" + number + "]");
				this.putValue(Action.ACCELERATOR_KEY,
						KeyStroke.getKeyStroke(number.charAt(0)));
			}
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (enterprise != null) {
				enterprise.doManeuver(index);
			}
		}
	}

	protected void recreateManeuversButtons() {
		JPanel p = getPanelManeuvers();
		p.removeAll();
		if (enterprise != null) {
			int count = enterprise.getNumberOfManeuvers();
			for (int i = 0; i < count; i++) {
				Action a = new ManeuverAction(i);
				JButton b = new JButton(a);
				b.setBackground(new Color(51, 0, 255));
				b.setForeground(new Color(255, 204, 0));
				b.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
						(KeyStroke) a.getValue(Action.ACCELERATOR_KEY),
						new Integer(i));
				b.getActionMap().put(new Integer(i), a);
				GridBagConstraints constraints = new GridBagConstraints();
				constraints.gridwidth = GridBagConstraints.REMAINDER;
				constraints.insets = new Insets(2, 2, 2, 2);
				p.add(b, constraints);
			}
		}
	}

	public void updateState() {
		if (enterprise != null) {
			getBtnShield().getModel().setSelected(enterprise.isShieldOn());
			getList().setListData(enterprise.getAttackTargets().toArray(new SpaceShip[enterprise.getAttackTargets().size()]));
			getTglbtnForward()
					.getModel()
					.setPressed(
							enterprise.getManualStearingEngine() == Enterprise.MODE_MANUAL_STEARING_ENGINE_ON
									&& enterprise.getEngineMode() == AutonomousMover.ENGINE_MODE_STAND_BY);
			getTglbtnForward()
			.getModel()
			.setSelected(
					enterprise.getManualStearingEngine() == Enterprise.MODE_MANUAL_STEARING_ENGINE_ON
							&& enterprise.getEngineMode() == AutonomousMover.ENGINE_MODE_STAND_BY);
			getTglbtnStop()
					.getModel()
					.setPressed(
							enterprise.getManualStearingEngine() == Enterprise.MODE_MANUAL_STEARING_ENGINE_OFF
									|| enterprise.getEngineMode() != AutonomousMover.ENGINE_MODE_STAND_BY);
			getTglbtnLeft()
					.getModel()
					.setPressed(
							enterprise.getManualStearingRotation() == Enterprise.MODE_MANUAL_STEARING_ROTATION_LEFT
									&& enterprise.getEngineMode() == AutonomousMover.ENGINE_MODE_STAND_BY);
			getTglbtnLeft()
					.getModel()
					.setPressed(
							enterprise.getManualStearingRotation() == Enterprise.MODE_MANUAL_STEARING_ROTATION_LEFT
									&& enterprise.getEngineMode() == AutonomousMover.ENGINE_MODE_STAND_BY);
			getTglbtnRight()
					.getModel()
					.setPressed(
							enterprise.getManualStearingRotation() == Enterprise.MODE_MANUAL_STEARING_ROTATION_RIGHT
									&& enterprise.getEngineMode() == AutonomousMover.ENGINE_MODE_STAND_BY);
			getBtnStandBy()
					.getModel()
					.setPressed(
							enterprise.getManualStearingRotation() == Enterprise.MODE_MANUAL_STEARING_ROTATION_OFF
									&& enterprise.getManualStearingEngine() == Enterprise.MODE_MANUAL_STEARING_ENGINE_OFF
									&& enterprise.getEngineMode() == AutonomousMover.ENGINE_MODE_STAND_BY);

		}
	}

	public JToggleButton getBtnShield() {
		return btnShield;
	}

	protected JButton getBtnAtackNearestKlingon() {
		return btnAtackNearestKlingon;
	}

	public JButton getBtnAtackAllKlingon() {
		return btnAtackAllKlingon;
	}

	public JList<SpaceShip> getList() {
		return list;
	}

	public JToggleButton getBtnStandBy() {
		return btnStandBy;
	}

	public JToggleButton getBtnRescue() {
		return btnRescue;
	}

	public JButton getTglbtnLeft() {
		return tglbtnLeft;
	}

	public JButton getTglbtnRight() {
		return tglbtnRight;
	}

	public JButton getTglbtnForward() {
		return tglbtnForward;
	}

	public JToggleButton getTglbtnStop() {
		return tglbtnStop;
	}

	public JPanel getPanelManeuvers() {
		return panelManeuvers;
	}

	public static void main(String[] args) {
		new EnterpriseControlPanel().setVisible(true);

	}

	protected JPanel getPanelMove() {
		if (panelMove == null) {
			panelMove = new JPanel();
			panelMove.setForeground(new Color(255, 204, 0));
			panelMove.setBackground(Color.BLACK);
			panelMove.setBorder(new TitledBorder(new LineBorder(new Color(255, 204, 0), 5, true), "Move", TitledBorder.LEADING, TitledBorder.TOP, null, new Color(255, 204, 0)));
			GridBagLayout gbl_panelMove = new GridBagLayout();
			panelMove.setLayout(gbl_panelMove);

			btnStandBy = new JToggleButton("Stand by [S]");
			btnStandBy.setBackground(new Color(51, 0, 255));
			btnStandBy.setForeground(new Color(255, 204, 0));
			btnStandBy.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise.setModeStandBy();
					}
				}
			});
			btnStandBy.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_S, 0, true),
					"STAND_BY");
			btnStandBy.getActionMap().put("STAND_BY",
					new AbstractAction() {
						private static final long serialVersionUID = -4001546317986859307L;

						@Override
						public void actionPerformed(ActionEvent e) {
							btnStandBy.doClick();
						}
					});
			buttonGroupMove.add(btnStandBy);
			GridBagConstraints gbc_btnStandBy = new GridBagConstraints();
			gbc_btnStandBy.gridx = GridBagConstraints.RELATIVE;
			gbc_btnStandBy.gridy = GridBagConstraints.RELATIVE;
			gbc_btnStandBy.anchor = GridBagConstraints.WEST;
			gbc_btnStandBy.insets = new Insets(0, 0, 5, 5);
			panelMove.add(btnStandBy, gbc_btnStandBy);
			btnRescue = new JToggleButton("Rescue [R]");
			btnRescue.setBackground(new Color(51, 0, 255));
			btnRescue.setForeground(new Color(255, 204, 0));
			btnRescue.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise.setModeRescue();
					}
				}
			});
			btnRescue.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_R, 0, true),
					"RESCUE");
			btnRescue.getActionMap().put("RESCUE",
					new AbstractAction() {
						private static final long serialVersionUID = 6746192921091626147L;

						@Override
						public void actionPerformed(ActionEvent e) {
							btnRescue.doClick();
						}
					});
			buttonGroupMove.add(btnRescue);
			GridBagConstraints gbc_btnRescue = new GridBagConstraints();
			gbc_btnRescue.gridx = GridBagConstraints.RELATIVE;
			gbc_btnRescue.gridy = GridBagConstraints.RELATIVE;
			gbc_btnRescue.anchor = GridBagConstraints.WEST;
			gbc_btnRescue.insets = new Insets(0, 0, 5, 5);
			panelMove.add(btnRescue, gbc_btnRescue);
			btnRescue.requestFocusInWindow();

			panelManeuvers = new JPanel();
			panelManeuvers.setBackground(Color.BLACK);
			panelManeuvers.setLayout(new GridBagLayout());
			panelManeuvers.setBorder(new TitledBorder(new LineBorder(new Color(255, 204, 0), 5, true), "Maneuvers", TitledBorder.LEADING, TitledBorder.TOP, null, new Color(255, 204, 0)));
			GridBagConstraints gbc_panelManeuvers = new GridBagConstraints();
			gbc_panelManeuvers.gridheight = 2;
			gbc_panelManeuvers.fill = GridBagConstraints.BOTH;
			gbc_panelManeuvers.weightx = 1;
			gbc_panelManeuvers.weighty = 1;
			gbc_panelManeuvers.insets = new Insets(0, 0, 5, 0);
			gbc_panelManeuvers.gridx = GridBagConstraints.RELATIVE;
			gbc_panelManeuvers.gridy = GridBagConstraints.RELATIVE;
			gbc_panelManeuvers.gridwidth = GridBagConstraints.REMAINDER;
			gbc_panelManeuvers.gridheight = 2;
			gbc_panelManeuvers.anchor = GridBagConstraints.NORTHWEST;
			panelMove.add(panelManeuvers, gbc_panelManeuvers);

			GridBagConstraints gbc_panelManualcontrol = new GridBagConstraints();
			gbc_panelManualcontrol.insets = new Insets(0, 0, 0, 5);
			gbc_panelManualcontrol.fill = GridBagConstraints.NONE;
			gbc_panelManualcontrol.gridx = 0;
			gbc_panelManualcontrol.gridy = 1;
			gbc_panelManualcontrol.gridwidth = 2;
			gbc_panelManualcontrol.weighty = 2;
			gbc_panelManualcontrol.anchor = GridBagConstraints.NORTH;
			panelMove.add(getPanelManualControl(), gbc_panelManualcontrol);

		}
		return panelMove;
	}

	protected JPanel getPanelManualControl() {
		if (panelManualControl == null) {
			panelManualControl = new JPanel();
			panelManualControl.setBackground(Color.BLACK);

			GridBagLayout gbl_panelManualcontrol = new GridBagLayout();
			panelManualControl.setLayout(gbl_panelManualcontrol);

			tglbtnForward = new JButton("^");
			tglbtnForward.setBackground(new Color(51, 0, 255));
			tglbtnForward.setForeground(new Color(255, 204, 0));
			tglbtnForward.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0, true),
					"FORWARD_OFF");
			tglbtnForward.getActionMap().put("FORWARD_OFF",
					new AbstractAction() {
						private static final long serialVersionUID = -5971047444262662297L;

						@Override
						public void actionPerformed(ActionEvent e) {
							if (enterprise != null) {
								enterprise.engineOn(false);
							}
						}
					});
			tglbtnForward.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0, false),
					"FORWARD_ON");
			tglbtnForward.getActionMap().put("FORWARD_ON",
					new AbstractAction() {
						private static final long serialVersionUID = 2149776589014735903L;

						@Override
						public void actionPerformed(ActionEvent e) {
							if (enterprise != null) {
								enterprise.engineOn(true);
							}
						}
					});
			tglbtnForward.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					if (enterprise != null) {
						enterprise.engineOn(getTglbtnForward().getModel()
								.isPressed());
					}
				}
			});
			GridBagConstraints gbc_tglbtnForward = new GridBagConstraints();
			gbc_tglbtnForward.gridwidth = 0;
			gbc_tglbtnForward.insets = new Insets(0, 0, 5, 5);
			gbc_tglbtnForward.gridx = GridBagConstraints.RELATIVE;
			gbc_tglbtnForward.gridy = GridBagConstraints.RELATIVE;
			panelManualControl.add(tglbtnForward, gbc_tglbtnForward);

			tglbtnLeft = new JButton("<");
			tglbtnLeft.setBackground(new Color(51, 0, 255));
			tglbtnLeft.setForeground(new Color(255, 204, 0));
			GridBagConstraints gbc_tglbtnLeft = new GridBagConstraints();
			gbc_tglbtnLeft.anchor = GridBagConstraints.NORTHWEST;
			gbc_tglbtnLeft.insets = new Insets(0, 0, 0, 5);
			gbc_tglbtnLeft.gridx = GridBagConstraints.RELATIVE;
			gbc_tglbtnLeft.gridy = GridBagConstraints.RELATIVE;
			panelManualControl.add(tglbtnLeft, gbc_tglbtnLeft);
			tglbtnLeft.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0, true),
					"LEFT_OFF");
			tglbtnLeft.getActionMap().put("LEFT_OFF", new AbstractAction() {
				private static final long serialVersionUID = 2753536246184872673L;

				@Override
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise
								.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_OFF);
					}
				}
			});
			tglbtnLeft.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0, false),
					"LEFT_ON");
			tglbtnLeft.getActionMap().put("LEFT_ON", new AbstractAction() {
				private static final long serialVersionUID = -4206481821082105967L;

				@Override
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise
								.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_LEFT);
					}
				}
			});
			tglbtnLeft.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					if (enterprise != null) {
						if (getTglbtnLeft().getModel().isPressed()) {
							enterprise
									.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_LEFT);
						} else {
							enterprise
									.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_OFF);
						}
					}
				}
			});

			tglbtnStop = new JToggleButton("Stop");
			tglbtnStop.setBackground(new Color(51, 0, 255));
			tglbtnStop.setForeground(new Color(255, 204, 0));
			GridBagConstraints gbc_tglbtnStop = new GridBagConstraints();
			gbc_tglbtnStop.anchor = GridBagConstraints.NORTHWEST;
			gbc_tglbtnStop.insets = new Insets(0, 0, 0, 5);
			gbc_tglbtnStop.gridx = GridBagConstraints.RELATIVE;
			gbc_tglbtnStop.gridy = GridBagConstraints.RELATIVE;
			panelManualControl.add(tglbtnStop, gbc_tglbtnStop);
			tglbtnStop.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise.engineOn(false);
					}
				}
			});
			buttonGroupEngine.add(tglbtnStop);

			tglbtnRight = new JButton(">");
			tglbtnRight.setBackground(new Color(51, 0, 255));
			tglbtnRight.setForeground(new Color(255, 204, 0));
			GridBagConstraints gbc_tglbtnRight = new GridBagConstraints();
			gbc_tglbtnRight.anchor = GridBagConstraints.NORTHWEST;
			gbc_tglbtnRight.gridx = GridBagConstraints.RELATIVE;
			gbc_tglbtnRight.gridy = GridBagConstraints.RELATIVE;
			panelManualControl.add(tglbtnRight, gbc_tglbtnRight);
			tglbtnRight.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0, true),
					"RIGHT_OFF");
			tglbtnRight.getActionMap().put("RIGHT_OFF", new AbstractAction() {
				private static final long serialVersionUID = 1583218432376545287L;

				@Override
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise
								.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_OFF);
					}
				}
			});
			tglbtnRight.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0, false),
					"RIGHT_ON");
			tglbtnRight.getActionMap().put("RIGHT_ON", new AbstractAction() {
				private static final long serialVersionUID = 7936147047355219265L;

				@Override
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise
								.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_RIGHT);
					}
				}
			});
			tglbtnRight.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					if (enterprise != null) {
						if (getTglbtnRight().getModel().isPressed()) {
							enterprise
									.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_RIGHT);
						} else {
							enterprise
									.setRotateDirection(Enterprise.MODE_MANUAL_STEARING_ROTATION_OFF);
						}
					}
				}
			});

		}
		return panelManualControl;
	}
	protected JPanel getPanelAttack() {
		if(panelAttack == null){
			panelAttack = new JPanel();
			panelAttack.setForeground(new Color(255, 204, 0));
			panelAttack.setBackground(Color.BLACK);
			panelAttack.setBorder(new TitledBorder(new LineBorder(new Color(255, 204, 0), 5, true), "Attack", TitledBorder.LEADING, TitledBorder.TOP, null, new Color(255, 204, 0)));
			GridBagLayout gbl_panelAttack = new GridBagLayout();
			panelAttack.setLayout(gbl_panelAttack);

			JScrollPane scrollPane = new JScrollPane();
			GridBagConstraints gbc_scrollPane = new GridBagConstraints();
			gbc_scrollPane.anchor = GridBagConstraints.CENTER;
			gbc_scrollPane.fill = GridBagConstraints.BOTH;
			gbc_scrollPane.weightx = 1;
			gbc_scrollPane.weighty = 1;
			gbc_scrollPane.gridx = GridBagConstraints.RELATIVE;
			gbc_scrollPane.gridy = GridBagConstraints.RELATIVE;
			gbc_scrollPane.gridheight = GridBagConstraints.REMAINDER;
			panelAttack.add(scrollPane, gbc_scrollPane);

			list = new JList<SpaceShip>();
			scrollPane.setViewportView(list);

			btnAtackNearestKlingon = new JButton("Atack Nearest Klingon Cruiser [N]");
			btnAtackNearestKlingon.setBackground(new Color(204, 102, 51));
			btnAtackNearestKlingon.setForeground(new Color(255, 204, 0));
			btnAtackNearestKlingon.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise.attackNearestKlingon();
					}

				}
			});
			btnAtackNearestKlingon.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_N, 0, true),
					"ATTACK_NEAREST");
			btnAtackNearestKlingon.getActionMap().put("ATTACK_NEAREST",
					new AbstractAction() {
						private static final long serialVersionUID = -8438579512564707030L;

						@Override
						public void actionPerformed(ActionEvent e) {
							btnAtackNearestKlingon.doClick();
						}
					});
			GridBagConstraints gbc_btnAtackNearestKlingon = new GridBagConstraints();
			gbc_btnAtackNearestKlingon.weighty = 1.0;
			gbc_btnAtackNearestKlingon.insets = new Insets(0, 0, 0, 5);
			gbc_btnAtackNearestKlingon.fill = GridBagConstraints.BOTH;
			gbc_btnAtackNearestKlingon.gridx = GridBagConstraints.RELATIVE;
			gbc_btnAtackNearestKlingon.gridy = GridBagConstraints.RELATIVE;
			gbc_btnAtackNearestKlingon.gridwidth = GridBagConstraints.REMAINDER;
			panelAttack.add(btnAtackNearestKlingon, gbc_btnAtackNearestKlingon);

			btnStopAttack = new JButton("Halt Attack [H]");
			btnStopAttack.setBackground(new Color(204, 102, 51));
			btnStopAttack.setForeground(new Color(255, 204, 0));
			btnStopAttack.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise.stopAttack();
					}
				}
			});
			btnStopAttack.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_H, 0, true),
					"ATTACK_STOP");
			btnStopAttack.getActionMap().put("ATTACK_STOP",
					new AbstractAction() {
						private static final long serialVersionUID = -7579646470093927763L;

						@Override
						public void actionPerformed(ActionEvent e) {
							btnStopAttack.doClick();
						}
					});

			btnAtackAllKlingon = new JButton("Atack All Klingon Ships [A]");
			btnAtackAllKlingon.setBackground(new Color(204, 102, 51));
			btnAtackAllKlingon.setForeground(new Color(255, 204, 0));
			btnAtackAllKlingon.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (enterprise != null) {
						enterprise.attackAllKlingon();
					}
				}
			});
			btnAtackAllKlingon.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke(KeyEvent.VK_A, 0, true),
					"ATTACK_ALL");
			btnAtackAllKlingon.getActionMap().put("ATTACK_ALL",
					new AbstractAction() {
						private static final long serialVersionUID = -3833399461175659458L;

						@Override
						public void actionPerformed(ActionEvent e) {
							btnAtackAllKlingon.doClick();
						}
					});
			
			GridBagConstraints gbc_btnAtackAllKlingon = new GridBagConstraints();
			gbc_btnAtackAllKlingon.weighty = 1.0;
			gbc_btnAtackAllKlingon.insets = new Insets(0, 0, 0, 5);
			gbc_btnAtackAllKlingon.gridx = GridBagConstraints.RELATIVE;
			gbc_btnAtackAllKlingon.gridy = GridBagConstraints.RELATIVE;
			gbc_btnAtackAllKlingon.gridwidth = GridBagConstraints.REMAINDER;
			gbc_btnAtackAllKlingon.fill = GridBagConstraints.BOTH;
			panelAttack.add(btnAtackAllKlingon, gbc_btnAtackAllKlingon);
			
			GridBagConstraints gbc_btnStopAttack = new GridBagConstraints();
			gbc_btnStopAttack.weighty = 1.0;
			gbc_btnStopAttack.insets = new Insets(0, 0, 0, 5);
			gbc_btnStopAttack.gridx = GridBagConstraints.RELATIVE;
			gbc_btnStopAttack.gridy = GridBagConstraints.RELATIVE;
			gbc_btnStopAttack.fill = GridBagConstraints.BOTH;
			gbc_btnStopAttack.gridwidth = GridBagConstraints.REMAINDER;
			panelAttack.add(btnStopAttack, gbc_btnStopAttack);

			
		}
		return panelAttack;
	}
}
