import java.util.ArrayList;
import java.util.List;

import greenfoot.Actor;
import greenfoot.World;
import greenfoot.util.Location;


public class GeoTools {

	public static int getAngelTo(double x, double y, double toX, double toY){
		return (int) Math.round(getPreciseAngelTo(x, y, toX, toY));
	}
	
	public static double getPreciseAngelTo(double x, double y, double toX, double toY){
		double vx = toX - x;
		double vy = toY - y;
		double length = Math.sqrt(vx * vx + vy * vy);
		double deg = Math.toDegrees(Math.asin(vy / length));
		if (vx < 0) {
			if(deg < 0){
				deg = 180 + Math.abs(deg);
			} else {
				deg = 180  - Math.abs(deg);
			}
		}
		if (deg < 0) {
			deg += 360;
		}
		return deg;
	}
	
	public static int normalizeAngle(int a){
		if(a >= 360){
			a -= 360;
		}
		if(a < 0){
			a += 360;
		}
		return a;
	}

	public static ExactLocation move(ExactLocation l, double angle, double distance){
		double dx = Math.cos(Math.toRadians(angle))*distance;
		double dy = Math.sin(Math.toRadians(angle))*distance;
		l.add(dx, dy);
		return l;
	}

	public static Location move(Location l, int angle, int distance, World w){
		double dx = Math.cos(Math.toRadians(angle))*distance;
		double dy = Math.sin(Math.toRadians(angle))*distance;
		l.add((int)dx, (int)dy);
		return l;
	}
	
	public static Location normalize(Location l, World w){
		if(l.getX() > w.getWidth()){
			l.setX(w.getWidth());
		}
		if(l.getX() < 0){
			l.setX(0);
		}
		if(l.getY() > w.getHeight()){
			l.setY(w.getHeight());
		}
		if(l.getY() < 0){
			l.setY(0);
		}
		return l;
	}
	
	public static double getDistance(double x1, double y1, double x2, double y2){
		return Math.sqrt( Math.pow(x1-x2, 2) + Math.pow(y1-y2, 2) );
	}

	public static double getDistance(Actor a1, Actor a2){
		return getDistance(a1.getX(), a1.getY(), a2.getX(), a2.getY());
	}
	public static double getDistance(Location a1, Location a2){
		return getDistance(a1.getX(), a1.getY(), a2.getX(), a2.getY());
	}

	public static double getDistance(ExactLocation a1, ExactLocation a2){
		return getDistance(a1.getX(), a1.getY(), a2.getX(), a2.getY());
	}
	
	public static Location move(Location l, double angle, double distance){
        double radians = Math.toRadians(angle);
        double dx = Math.cos(radians) * distance;
        double dy = Math.sin(radians) * distance;
        l.setX((int)(l.getX() + dx));
        l.setY((int)(l.getY() + dy));
        return l;
	}
	
	public static Location rotateTranslate(Location l, double transX, double transY, int rot){
		double a = Math.toRadians(rot);
		double newX = l.getX()*Math.cos(a) - l.getY()*Math.sin(a);
		double newY = l.getX()*Math.sin(a) + l.getY()*Math.cos(a);
		l.setX((int)(newX + transX));
		l.setY((int)(newY + transY));
		return l;
	}
	
	public static ArrayList<Location> rotateTranslate(ArrayList<Location> list, double transX, double transY, int rot){
		ArrayList<Location> newList = new ArrayList<Location>(list.size());
		for(Location l : list){
			newList.add(rotateTranslate(new Location(l.getX(), l.getY()), transX, transY, rot));
		}
		return newList;
	}
	public static ArrayList<Location> rotateTranslateNormalize(List<Location> list, double transX, double transY, int rot, World w){
		ArrayList<Location> newList = new ArrayList<Location>(list.size());
		for(Location l : list){
			newList.add(normalize(rotateTranslate(new Location(l.getX(), l.getY()), transX, transY, rot), w));
		}
		return newList;
	}
}
