import greenfoot.util.Location;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

/**
 * Write a description of class KlingonCruiser here.
 * 
 * @author (your name)
 * @version (a version number or a date)
 */
public class KlingonCruiser extends SpaceShip {
	
	protected static int instanceCounter = 0;
	
	protected final int KLINGON_MODE_PATROL = 0;
	protected final int KLINGON_MODE_ATTACK = 1;

	protected int battleMode = KLINGON_MODE_PATROL;

	protected Random rand = new Random();

	/**
	 * Act - do whatever the KlingonCruiser wants to do. This method is called
	 * whenever the 'Act' or 'Run' button gets pressed in the environment.
	 */
	public void act() {
		super.act();
		switch (battleMode) {
		case KLINGON_MODE_PATROL:
			if (hasEnterpriseSwitchOnShields()) {
				List<Enterprise> enterprises = getAllEnterprises();
				for (Enterprise e : enterprises) {
					Location l = GeoTools
							.move(new Location(e.getX(), e.getY()),
									e.getRotation(), 50, getWorld());
					warningPlaces.add(l);
				}
			}
			if (hasEnterpriseSwitchOnEngine() || enterpriseAttack()) {
				switchToAttackMode();
			}
			break;
		case KLINGON_MODE_ATTACK:
			List<Enterprise> enterprises = getAllEnterprises();
			if (enterprises.size() == 0) {
				switchToPatrolMode();
			}
			break;
		}
	}

	@Override
	protected void attack() {
		if (warningPlaces.size() > 0) {
			if (availableShooters() > 0) {

				while (availableShooters() > 0 && warningPlaces.size() > 0) {
					Location target = warningPlaces.get(0);
					warningPlaces.remove(target);
					ShooterDescription sd = getBestShooter(target.getX(),
							target.getY());
					shotTo(target.getX(), target.getY(), sd);
				}
			}
		}
		super.attack();
	}

	protected ArrayList<Location> warningPlaces = new ArrayList<Location>();
	protected int warningPlacesIndex = 0;

	protected void attackModeWarnningShot() {

	}

	protected boolean enterpriseShieldOn = true;

	public boolean hasEnterpriseSwitchOnShields() {
		List<Enterprise> enterprises = getAllEnterprises();
		boolean atLeastOneUp = false;
		for (Enterprise e : enterprises) {
			atLeastOneUp = atLeastOneUp || e.isShieldOn();
		}
		if (enterpriseShieldOn && !atLeastOneUp) {
			enterpriseShieldOn = false;
			return false;
		}
		if (!enterpriseShieldOn && atLeastOneUp) {
			enterpriseShieldOn = true;
			return true;
		}
		return false;
	}

	protected HashMap<Enterprise, Location> enterpriseLastLocations = new HashMap<Enterprise, Location>();

	public boolean hasEnterpriseSwitchOnEngine() {
		List<Enterprise> enterprises = getAllEnterprises();

		boolean engineOn = false;
		for (Enterprise e : enterprises) {
			if (enterpriseLastLocations.containsKey(e)) {
				Location oldLocation = enterpriseLastLocations.get(e);
				if (oldLocation.getX() != e.getX()
						|| oldLocation.getY() != e.getY()) {
					engineOn = true;
				}
			} else {
				enterpriseLastLocations
						.put(e, new Location(e.getX(), e.getY()));
			}
		}
		return engineOn;
	}

	@SuppressWarnings("unchecked")
	private List<Enterprise> getAllEnterprises() {
		return (List<Enterprise>) getObjectsInRange(
				1000, Enterprise.class);
	}
	
	public boolean enterpriseAttack(){
		@SuppressWarnings("unchecked")
		List<Torpedo> torpedos = getWorld().getObjects(Torpedo.class);
		for(Torpedo t : torpedos){
			if(!t.isKlingon()){
				return true;
			}
		}
		return false;
	}

	@Override
	protected void destinationReached() {
		super.destinationReached();
		switch (battleMode) {
		case KLINGON_MODE_PATROL:
			destinationReachedInPatrolMode();
			break;
		case KLINGON_MODE_ATTACK:
			destinationReachedInAttackMode();
			break;
		}
	}

	protected void switchToAttackMode() {
		battleMode = KLINGON_MODE_ATTACK;
		canStop = false;
		List<Enterprise> enterprises = getAllEnterprises();
		addAttackTargets(enterprises);
		attackModePathIndex = 0;
		destinationReachedInAttackMode();
	}

	protected void switchToPatrolMode() {
		battleMode = KLINGON_MODE_PATROL;
		canStop = true;
		destinationReachedInPatrolMode();
	}

	protected int attackModePathIndex = 0;

	protected void destinationReachedInAttackMode() {
		if (attackTargets.size() > 0) {
			SpaceShip s = attackTargets.get(0);
			if(s.getWorld() == null){
				return;
			}
			Location l = new Location(s.getX(), s.getY());
			switch (attackModePathIndex) {
			case 0:
				l = GeoTools.move(l, s.getRotation(), 100, getWorld());
				GeoTools.normalize(l, getWorld());
				setDestination(l.getX(), l.getY());
				break;
			case 1:
				l = GeoTools.move(l, GeoTools.normalizeAngle(s.getRotation() + 40), 450, getWorld());
				GeoTools.normalize(l, getWorld());
				setDestination(l.getX(), l.getY());
				break;
			case 2:
				l = GeoTools.move(l, s.getRotation(), 300, getWorld());
				GeoTools.normalize(l, getWorld());
				setDestination(l.getX(), l.getY());
				break;
			case 3:
				l = GeoTools.move(l, GeoTools.normalizeAngle(s.getRotation() - 40), 450, getWorld());
				GeoTools.normalize(l, getWorld());
				setDestination(l.getX(), l.getY());
				break;
			}
			attackModePathIndex = (attackModePathIndex+1) % 4;
		}
	}

	protected void destinationReachedInPatrolMode() {
		int border = ((SpaceMap) getWorld()).getBorder();
		int x = rand.nextInt(getWorld().getWidth() - 40) + 20;
		int y = rand.nextInt((getWorld().getHeight() - border) / 3) + border;
		setDestination(x, y);
	}

	public KlingonCruiser() {
		super();
		instanceCounter++;
		setName("KlingonCruiser "+ instanceCounter);
		// ArrayList<Location> path = new ArrayList<Location>();
		// path.add(new Location(300,300));
		// path.add(new Location(400,200));
		// path.add(new Location(300,400));
		// path.add(new Location(200,500));
		// path.add(new Location(300,300));
		// moveByPath(path);
	}

	@Override
	public void youWasAdded() {
		super.youWasAdded();
		switchToPatrolMode();
	}

	@Override
	protected Torpedo torpedoFactoryMethod() {
		Torpedo t = new Torpedo(10, 6, 2, true);
		return t;
	}

	@Override
	protected Explosion explosionFctoryMethod() {
		return new Explosion(true);
	}
}
