import greenfoot.Actor;
import greenfoot.GreenfootImage;
import greenfoot.World;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

// (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class SpaceShip here.
 * 
 * @author (your name)
 * @version (a version number or a date)
 */
public class SpaceShip extends LabeledActor {
	protected String name;

	protected int shooterAngleRange = 40;
	protected int shooterSpeed = 100;
	protected int frontShooterCount = 2;
	protected int rearShooterCount = 2;
	protected int[] frontShooterCoolDownTime;
	protected int[] rearShooterCoolDownTime;
	protected int coolDownTime = 120;
	protected int globalShooterCoolDownTime = 8;
	protected int frontShooterGlobalCoolDown = 0;
	protected int rearShooterGlobalCoolDown = 0;
	protected int shield = 100;
	protected boolean shieldOn = true;

	protected double shieldRadius;
	protected GreenfootImage icon = null;

	protected ArrayList<SpaceShip> attackTargets = new ArrayList<SpaceShip>();

	protected class ShooterDescription {
		public int shooterIndex;
		public boolean frontShooter;
	}

	/**
	 * Act - do whatever the SpaceShip wants to do. This method is called
	 * whenever the 'Act' or 'Run' button gets pressed in the environment.
	 */

	@Override
	protected void addedToWorld(World world1) {
		super.addedToWorld(world1);
		updateLabel();
	}

	public SpaceShip() {
		super();
		constructImage();
		updateImage();
		initShooter();
	}
	
	@Override
	public void act() {
		super.act();
		coolDownShooters();
		attack();
	}

	private int attackIndex = 0;
	protected void attack() {
		if (attackTargets.size() > 0) {
			if (availableShooters() > 0) {
				
				while (availableShooters() > 0) {
					SpaceShip target = attackTargets.get(attackIndex
							% attackTargets.size());
					if (target.getWorld() == null) {
						removeAttackTarget(target);
						if (attackTargets.size() == 0) {
							return;
						}
						continue;
					}
					ShooterDescription sd = getBestShooter(target.getX(), target.getY());
					shotTo(target, sd);
					attackIndex++;
					if(attackIndex == Integer.MAX_VALUE){
						attackIndex = 0;
					}
				}
			}
		}
	}

	protected ShooterDescription getBestShooter(int x, int y) {
		if (availableShooters() <= 0) {
			return null;
		}
		int angle = getAngelTo(x, y);
		int[] preferedShooters = frontShooterCoolDownTime;
		int[] backupShooters = rearShooterCoolDownTime;
		int prefferedGlobalCoolDownTime = frontShooterGlobalCoolDown;
		int backupGlobalCoolDownTime = rearShooterGlobalCoolDown;
		boolean prefeerFront = true;
		if (angle > 90 || angle < 270) {
			prefeerFront = false;
			preferedShooters = rearShooterCoolDownTime;
			backupShooters = frontShooterCoolDownTime;
			prefferedGlobalCoolDownTime = rearShooterGlobalCoolDown;
			backupGlobalCoolDownTime = frontShooterGlobalCoolDown;
		}
		ShooterDescription sd = new ShooterDescription();
		boolean found = false;
		if (prefferedGlobalCoolDownTime == 0) {
			for (int i = 0; i < preferedShooters.length; i++) {
				if (preferedShooters[i] == 0) {
					sd.frontShooter = prefeerFront;
					sd.shooterIndex = i;
					found = true;
					break;
				}
			}
		}
		if (!found) {
			if (backupGlobalCoolDownTime == 0) {
				for (int i = 0; i < backupShooters.length; i++) {
					if (backupShooters[i] == 0) {
						sd.frontShooter = !prefeerFront;
						sd.shooterIndex = i;
						found = true;
						break;
					}
				}
			}
		}
		return sd;
	}

	protected void initShooter() {
		frontShooterCoolDownTime = new int[frontShooterCount];
		rearShooterCoolDownTime = new int[rearShooterCount];
		Arrays.fill(frontShooterCoolDownTime, 0);
		Arrays.fill(rearShooterCoolDownTime, 0);
	}

	protected void coolDownShooters() {
		if (frontShooterGlobalCoolDown > 0) {
			frontShooterGlobalCoolDown--;
		}
		if (rearShooterGlobalCoolDown > 0) {
			rearShooterGlobalCoolDown--;
		}
		for (int i = 0; i < frontShooterCount; i++) {
			if (frontShooterCoolDownTime[i] > 0) {
				frontShooterCoolDownTime[i]--;
			}
		}
		for (int i = 0; i < rearShooterCount; i++) {
			if (rearShooterCoolDownTime[i] > 0) {
				rearShooterCoolDownTime[i]--;
			}
		}
	}

	protected int availableShooters() {
		int count = 0;
		if (frontShooterGlobalCoolDown == 0) {
			for (int i = 0; i < frontShooterCount; i++) {
				if (frontShooterCoolDownTime[i] == 0) {
					count++;
				}
			}
		}
		if (rearShooterGlobalCoolDown == 0) {
			for (int i = 0; i < rearShooterCount; i++) {
				if (rearShooterCoolDownTime[i] == 0) {
					count++;
				}
			}
		}
		return count;
	}

	protected void constructImage() {
		icon = getImage();
		shieldRadius = Math.sqrt(Math.pow(icon.getWidth(), 2)
				+ Math.pow(icon.getHeight(), 2)) / 2;
		GreenfootImage img = new GreenfootImage((int) (shieldRadius * 2 + 5),
				(int) (shieldRadius * 2 + 5));
		setImage(img);

	}

	protected void updateImage() {
		Graphics2D g = (Graphics2D) getImage().getAwtImage().getGraphics();
		g.setBackground(new Color(255, 255, 255, 0));
		g.clearRect(0, 0, getImage().getWidth(), getImage().getHeight());
		if (shieldOn) {
			int d = shield / 20;
			if(d > 20){
				d = 20;
			}
			g.setStroke(new BasicStroke(d));
			g.setColor(new Color(0, 0, 255, 150));
			g.drawOval(5, 5, getImage().getWidth() - 10,
					getImage().getHeight() - 10);
		}
		g.drawImage(icon.getAwtImage(), (getImage().getWidth() - icon
				.getWidth()) / 2,
				(getImage().getHeight() - icon.getHeight()) / 2,
				new ImageObserver() {

					@Override
					public boolean imageUpdate(Image img, int infoflags, int x,
							int y, int width, int height) {
						// TODO Auto-generated method stub
						return false;
					}
				});
	}

	protected void updateLabel() {
		if (getLabel() != null) {
			getLabel().setTextPattern(name);
		}
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
		updateLabel();
	}

	protected Torpedo torpedoFactoryMethod(){
		return new Torpedo();
	}
	protected Torpedo createTorpedo(int x, int y, boolean front){
		Torpedo t = torpedoFactoryMethod();
		int shooterRotation = getRotation();
		int angelToTarget = getAngelTo(x, y);
		if (!front) {
			shooterRotation += 180;
			shooterRotation = GeoTools.normalizeAngle(shooterRotation);
		}
		int angleDiff = angelToTarget - shooterRotation;
		if (angleDiff > 0) {
			angleDiff = Math.min(shooterAngleRange / 2, angleDiff);
		} else {
			angleDiff = Math.max(-shooterAngleRange / 2, angleDiff);
		}
		getWorld().addObject(t, getX(), getY());
		t.setRotation(shooterRotation + angleDiff);
		return t;
	}
	
	protected boolean activeShooters(ShooterDescription sd){
		int[] activeShooter = frontShooterCoolDownTime;
		if (!sd.frontShooter) {
			activeShooter = rearShooterCoolDownTime;
			if(rearShooterGlobalCoolDown > 0){
				return false;
			}
		} else {
			if(frontShooterGlobalCoolDown > 0){
				return false;
			}
		}
		if (activeShooter[sd.shooterIndex] > 0) {
			return false;
		}
		activeShooter[sd.shooterIndex] = coolDownTime;
		if(sd.frontShooter){
			frontShooterGlobalCoolDown = globalShooterCoolDownTime;
		} else {
			rearShooterGlobalCoolDown = globalShooterCoolDownTime;
		}
		return true;
	}
	
	public void shotTo(Actor a, ShooterDescription sd) {
		if(activeShooters(sd)){
			Torpedo t = createTorpedo(a.getX(), a.getY(), sd.frontShooter);
			t.pursuite(a);
		}
	}
	public void shotTo(int x, int y, ShooterDescription sd) {
		if(activeShooters(sd)){
			Torpedo t = createTorpedo(x, y, sd.frontShooter);
			t.setDestination(x, y);
		}
	}

	public void hit(int power) {
		if (shieldOn && shield > 0) {
			shield = shield - power;
			if(shield < 0){
				shield = 0;
			}
			updateImage();
		} else {
			explode();
		}
	}

	protected Explosion explosionFctoryMethod(){
		return new Explosion();
	}
	protected void explode() {
		Explosion ex = explosionFctoryMethod();
		getWorld().addObject(ex, getX(), getY());
		getWorld().removeObject(this);
	}

	public boolean isShieldOn() {
		return shieldOn;
	}

	public void setShieldOn(boolean shieldOn) {
		this.shieldOn = shieldOn;
		updateImage();
	}

	public void clearAttackList() {
		attackTargets.clear();
	}

	public boolean addAttackTarget(SpaceShip e) {
		return attackTargets.add(e);
	}

	public boolean addAttackTargets(Collection<? extends SpaceShip> c) {
		return attackTargets.addAll(c);
	}
	public void removeAttackTarget(SpaceShip e) {
		attackTargets.remove(e);
	}

	@Override
	public String toString() {
		return getName();
	}

	@Override
	public void worldWasStarted() {
	}

	@Override
	public void worldWasStopped() {
	}

}
